<?php

namespace SenSai;

use FML\ManiaLink;
use FML\Controls\Frame;
use FML\Controls\Quad;
use FML\Controls\Labels\Label_Text;
use FML\Controls\Control;
use ManiaControl\ManiaControl;
use ManiaControl\Callbacks\Callbacks;
use ManiaControl\Callbacks\CallbackListener;
use ManiaControl\Callbacks\CallbackManager;
use ManiaControl\Callbacks\TimerListener;
use ManiaControl\Callbacks\TimerManager;
use ManiaControl\Plugins\Plugin;
use ManiaControl\Maps\MapManager;
use ManiaControl\Players\Player;
use ManiaControl\Players\PlayerManager;
use ManiaControl\Commands\CommandListener;

/**
 * Loadscreens Plugin
 *
 * @author Sen-Sai
 */

class Loadscreens implements CallbackListener, TimerListener, CommandListener, Plugin {
    /**
     * Constants
     */
    const PLUGIN_ID     = 45;
    const PLUGIN_VERSION = 1.3;
    const PLUGIN_NAME = 'Loadscreens';
    const PLUGIN_AUTHOR = 'G]R [SG-1]Sen-sai';

    // Loadscreen properties

    const MLID_LOADSCREENS     = 'Loadscreens.ID';
    const SETTING_LS_ACTIVATED = 'Loadscreens activated';
    const SETTING_LS_POSX      = 'Loadscreens Position: X';
    const SETTING_LS_POSY      = 'Loadscreens Position: Y';
    const SETTING_LS_WIDTH     = 'Loadscreens Size: Width';
    const SETTING_LS_HEIGHT    = 'Loadscreens Size: Height';
    const SETTING_LS_URL       = 'Loadscreens URL';
    const SETTING_LS_OFFSET    = 'Loadscreens EndMatch Delay';

	private $screen = array();
	private $cnt = 0;

    /**
     * Private Properties
     */
    /**
     * @var maniaControl $maniaControl
     */
    private $maniaControl = null;
	private $mapbegon = true;
	private $loadscreenOn = false;
	private $displaying = false;
	private $display = 0;

    /**
     * Prepares the Plugin
     *
     * @param ManiaControl $maniaControl
     * @return mixed
     */
    public static function prepare(ManiaControl $maniaControl) {
        //do nothing
    }

    /**
     * Load the plugin
     *
     * @param ManiaControl $maniaControl
     * @return bool
     */
    public function load(ManiaControl $maniaControl) {
        $this->maniaControl = $maniaControl;

        //Register callbacks

        $this->maniaControl->settingManager->initSetting($this, self::SETTING_LS_ACTIVATED, true);
        $this->maniaControl->settingManager->initSetting($this, self::SETTING_LS_URL, 'http://files.designburo.nl/sm/loadscreen.jpg');
        $this->maniaControl->settingManager->initSetting($this, self::SETTING_LS_OFFSET, 15);
		
		$file = $this->maniaControl->settingManager->getSetting($this, self::SETTING_LS_URL);
		$array = get_headers($file);
		$string = $array[0];
		if(!strpos($string,"200")) {
			
			$this->maniaControl->chat->sendInformation('The URL does not exist. Please check. Loadscreens disabled.', null, false);
			return;
		}
		$this->maniaControl->callbackManager->registerCallbackListener(CallbackManager::CB_MP_MODESCRIPTCALLBACK, $this, 'handleCallbacks');
		$this->maniaControl->callbackManager->registerCallbackListener(CallbackManager::CB_MP_MODESCRIPTCALLBACKARRAY, $this, 'handleCallbacks');
		$this->maniaControl->callbackManager->registerCallbackListener(PlayerManager::CB_PLAYERCONNECT, $this, 'preloadtoLogin');
		$this->maniaControl->commandManager->registerCommandListener('hidescreen', $this, 'hide', true);
		$this->maniaControl->commandManager->registerCommandListener('showscreen', $this, 'displayLoadscreens', true);
		
		$ext = substr(strrchr($file, "."), 1);
		if ($ext=="xml")
		{
			if ($xml = simplexml_load_string(file_get_contents($file)))
			{
				foreach($xml->loadscreen_image as $screen)
				{
					$this->screen[] = $screen;
				}
			}
			$this->cnt = count($this->screen);
			$this->maniaControl->chat->sendInformation('Got '.$this->cnt.' screens', null, false);
		}
		elseif ($ext=="jpg" || $ext=="png")
		{
			$this->cnt=1;
			$this->screen = $file;
		}
		else
		{
			$this->maniaControl->chat->sendInformation('The URL must be either .xml, .png or .jpg', null, false);
			$this->maniaControl->callbackManager->unregisterCallbackListener($this);
			return;	
		}
		$this->preload();
		$this->maniaControl->timerManager->registerTimerListening($this, 'handle1Second', 1000);
    }
	public function preloadtoLogin(player $player) {
        $pos_x = 128;
        $pos_y = 128;
		$pos_z = -2000;
        $width = 128;
        $height = 180;
		$manialink = new ManiaLink(self::MLID_LOADSCREENS);
		$frame = new Frame();
        $manialink->add($frame);
        $frame->setPosition($pos_x, $pos_y);
		$frame->setSize($width, $height);

		if ($this->cnt > 1)
		{
			foreach ($this->screen as $image)
			{
				$quad = new Quad();
        		$frame->add($quad);
        		$quad->setSize($width, $height);
        		$quad->setImage($image);
				$quad->setVisible(false);
			}
		}
		else
		{
			$image= $this->screen;
			$quad = new Quad();
			$frame->add($quad);
			$quad->setSize($width, $height);
			$quad->setImage($image);	
			$quad->setVisible(false);		
		}
		$this->maniaControl->manialinkManager->sendManialink($manialink,$player);
	}
	
	public function preload() {
		//$login=$player->login;
        $pos_x = 128;
        $pos_y = 128;
		$pos_z = -2000;
        $width = 128;
        $height = 180;
		$manialink = new ManiaLink(self::MLID_LOADSCREENS);
		$frame = new Frame();
        $manialink->add($frame);
        $frame->setPosition($pos_x, $pos_y);
		$frame->setSize($width, $height);
		if ($this->cnt > 1)
		{
			foreach ($this->screen as $image)
			{
				$quad = new Quad();
        		$frame->add($quad);
        		$quad->setSize($width, $height);
        		$quad->setImage($image);
				$quad->setVisible(false);
			}
		}
		else
		{
			$image= $this->screen;
			$quad = new Quad();
			$frame->add($quad);
			$quad->setSize($width, $height);
			$quad->setImage($image);	
			$quad->setVisible(false);		
		}
		$this->maniaControl->chat->sendInformation('Preloading screens to all.', null, false);
		$this->maniaControl->manialinkManager->sendManialink($manialink);
	}
	
    public function displayLoadscreens(){
		if(!$this->maniaControl->settingManager->getSetting($this, self::SETTING_LS_ACTIVATED))
		{ return; }

		
		$queuedMap = $this->maniaControl->getMapManager()
		                                ->getMapQueue()
		                                ->getNextMap();

		$requester = null;
		// if the nextmap is not a queued map, get it from map info
		if (!$queuedMap) {
			$map    = $this->maniaControl->getClient()
			                             ->getNextMapInfo();
			$name   = $map->name;
			$author = $map->author;
		} else {
			$requester = $queuedMap[0];
			$map       = $queuedMap[1];
			$name      = $map->name;
			$author    = $map->authorLogin;
		}		
		
        $pos_x = 0;
        $pos_y = 0;
        $width = 320;
        $height = 180;
		if($this->cnt > 1)
		{
			$nr=mt_rand(1,($this->cnt-1));
			$image  = $this->screen[$nr];

		}
		else
		{
        	$image  = $this->screen;
		}
		$ext = substr(strrchr($image, "."), 1);
        $manialink = new ManiaLink(self::MLID_LOADSCREENS);

        $frame = new Frame();
        $manialink->add($frame);
		//$manialink->setTimeout(10000);
        $frame->setSize($width, $height);
        $frame->setPosition($pos_x, $pos_y);

		$quad = new Quad();
		$frame->add($quad);
		$quad->setSize($width, $height);
		$quad->setImage($image);
		$quad->setZ(25.0);
		
		$lbl_loading = new Label_Text();
		$frame->add($lbl_loading);
		$lbl_loading->setPosition(120,85);
		$lbl_loading->setZ(30.0);
		$lbl_loading->setText("Now loading..");
		$lbl_loading->setTextSize(0.3);
		
		$lbl_mapname = new Label_Text();
		$frame->add($lbl_mapname);
		$lbl_mapname->setPosition(120,80);
		$lbl_mapname->setZ(30.0);
		$lbl_mapname->setText($name);
		$lbl_mapname->setTextSize(1.0);

		$lbl_mapauthor = new Label_Text();
		$frame->add($lbl_mapauthor);
		$lbl_mapauthor->setPosition(120,75);
		$lbl_mapauthor->setZ(30.0);
		$lbl_mapauthor->setText("by ".$author);
		$lbl_mapauthor->setTextSize(0.5);
		
		$label = new Label_Text();
		$frame->add($label);
		$label->setPosition(120,70);
		$label->setZ(30.0);
		$label->setText("Loadscreens");
		$label->setTextSize(1.0);
		$labeel = new Label_Text();
		$frame->add($labeel);
		$labeel->setPosition(120,65);
		$labeel->setText("by G]R [SG-1]Sen-Sai");
		$labeel->setTextSize(0.3);
		$labeel->setZ(30.0);
        
		
        $this->maniaControl->manialinkManager->sendManialink($manialink);
		$this->displaying = true;
    }

    public function hideScreen() {
		if($this->loadscreenOn)
		{
        	$emptyManialink = new ManiaLink(self::MLID_LOADSCREENS);
			$emptyManialink->removeChildren();
        	$this->maniaControl->manialinkManager->sendManialink($emptyManialink);
			$this->loadscreenOn = false;
			$this->displaying = false;
		}
    }
	
    public function hide() {
        	$emptyManialink = new ManiaLink(self::MLID_LOADSCREENS);
			$emptyManialink->removeChildren();
        	$this->maniaControl->manialinkManager->sendManialink($emptyManialink);
			$this->loadscreenOn = false;
    }


    public function handleOnEndMap() {
        if($this->maniaControl->settingManager->getSetting($this, self::SETTING_LS_ACTIVATED))
		{
			$this->displayLoadscreens();
        }
    }
	
	public function handle1Second() {
		if (!$this->loadscreenOn){ return; }
		if ($this->displaying) {return;}
		if( (int)time() >= $this->display )
		{
			$this->displayLoadscreens();
		}
	}

	public function handleCallbacks(array $callback) {
		$callbackName = $callback[1][0];
		switch($callbackName) {
			
			// ROYAL
		case 'LibXmlRpc_EndMap':
			$this->display = time() +  (int)($this->maniaControl->settingManager->getSetting($this, self::SETTING_LS_OFFSET));
			$this->loadscreenOn=true;
		break;
		case 'EndMap':
			$this->display = time() +  (int)($this->maniaControl->settingManager->getSetting($this, self::SETTING_LS_OFFSET));
			$this->loadscreenOn=true;
		break;
		case 'LibXmlRpc_BeginMap':
			$this->hideScreen();
		break;
		case 'BeginMap':
			$this->hideScreen();
		break;
		}
	}


    public function unload() {
        $this->hideScreen();
        $this->maniaControl->callbackManager->unregisterCallbackListener($this);
        unset($this->maniaControl);
    }

     /**
     * Get plugin id
     *
     * @return int
     */
    public static function getId() {
        return self::PLUGIN_ID;
    }

    /**
     * Get Plugin Name
     *
     * @return string
     */
    public static function getName() {
        return self::PLUGIN_NAME;
    }

    /**
     * Get Plugin Version
     *
     * @return float,,
     */
    public static function getVersion() {
        return self::PLUGIN_VERSION;
    }

    /**
     * Get Plugin Author
     *
     * @return string
     */
    public static function getAuthor() {
        return self::PLUGIN_AUTHOR;
    }

    /**
     * Get Plugin Description
     *
     * @return string
     */
    public static function getDescription() {
        return 'Displays a custom images during map changes.';
    }
}